/**
 * This is a very simple strongly typed event emitter class, see README.md
 * for more details.
 */
/** Checks if a type extends undefined, without a discriminator. */
declare type HasUndefined<T, C = [T extends undefined ? true : false]> = C extends [
    true
] ? true : C extends [false] ? false : true;
/** An emitter function for a ts-typed-events Event. */
export declare type Emitter<T = undefined> = HasUndefined<T> extends true ? Exclude<T, undefined> extends never ? () => boolean : (emitting?: T) => boolean : (emitting: T) => boolean;
/**
 * A ts-typed-events event to register callbacks on to be invoked when they are
 * emitted.
 */
export declare class Event<T = undefined> {
    /** All the current listeners for this event. */
    private listeners;
    /**
     * Marked as protected to discourage creation outside of
     * createEventEmitter().
     */
    protected constructor();
    /**
     * Attaches a callback to trigger on all emits for this event.
     *
     * @param callback - The callback to invoke on all emits.
     */
    on(callback: (data: T) => void): void;
    /**
     * Attaches a callback to trigger on only the first emit for this event.
     * After that event is emitted this callback will automatically be
     * removed.
     *
     * @param callback - The callback to invoke only the next time this event
     * emits, then that callback is removed from this event.
     */
    once(callback: (emitted: T) => void): void;
    /**
     * Attaches a callback to trigger on only the first emit for this event.
     *
     * Returns a promise that resolves with the emitted data the next time this
     * event is triggered (only once).
     *
     * @returns A promise that resolves with the emitted data the next time this
     * event is triggered (only once).
     */
    once(): Promise<T>;
    /**
     * Removes a callback from this event (regardless of once vs on).
     *
     * Returns true if a callback was removed, false otherwise.
     *
     * @param listener - The callback to remove.
     * @returns True if a callback was removed, false otherwise.
     */
    off(listener: ((emitted: T) => void) | Promise<T>): boolean;
    /**
     * Removes ALL callbacks from this event, regardless of once vs on.
     *
     * Returns the number of listeners removed.
     *
     * @returns The number of listeners removed.
     */
    offAll(): number;
}
/**
 * A tuple of both [event, emit] and {event, emit},
 * for you to consume however you desire.
 * The emitter for this event will only exist returned here, separate from the
 * event.
 */
export declare type EventAndEmitter<T, TEvent extends Event<T>> = readonly [
    TEvent,
    Emitter<T>
] & {
    event: TEvent;
    emit: Emitter<T>;
};
/** An emitter function with itself and its event as properties keyed on it. */
export declare type EventEmitter<T, TEvent extends Event<T>> = Emitter<T> & {
    event: TEvent;
    emit: Emitter<T>;
};
/**
 * Creates and returns a new Event and the emitter for that Event.
 *
 * @returns A tuple of the [event, emit] both keyed as an array and object.
 */
export declare function createEventEmitter<T = undefined>(): EventEmitter<T, Event<T>>;
/**
 * A specialized Event that holds a reference to its own emit function.
 * This allows any code with access to the Event to also trigger emits.
 */
export declare class PublicEvent<T> extends Event<T> {
    /**
     * Emits a value to all the listeners, triggering their callbacks.
     * Returns true if the event had listeners emitted to,
     * false otherwise.
     * Because this exists on the event, any code with access to this event
     * can trigger the callback for all listeners.
     *
     * @param emitting - If the Event has a type, this is the data of that type
     * to emit to all listeners. If no type (undefined) this argument should
     * be omitted.
     * @returns True if the event had listeners emitted to, false otherwise.
     */
    emit: Emitter<T>;
    /**
     * Creates a new PublicConstructor, with it's emit accessible as a member
     * function.
     */
    constructor();
}
/**
 * A tuple of both [event, emit] and {event, emit},
 * for you to consume however you desire.
 * The emitter can be considered optional, as the PublicEvent returned can self
 * emit. However this is exposed for API parody with the non Public version.
 *
 * @returns A tuple of the [event, emit] both keyed as an array and object.
 */
export declare function createPublicEventEmitter<T = undefined>(): EventEmitter<T, PublicEvent<T>>;
export {};
